#include <stdlib.h>
#include <pgw.h>
#include "ptr.h"

typedef struct _WindowTable {
    int id;
    PTRWindow* wp;
    struct _WindowTable* next;
} WindowTable;
WindowTable wtable={0, NULL, NULL};

typedef struct _DataTable {
    int id;
    PTRData* dp;
    struct _DataTable* next;
} DataTable;
DataTable dtable={0, NULL, NULL};



char* scopy(char* str, int len){
    int i;
    char* ret;
    if(len<1) return NULL;
    ret=(char*)malloc((len+1)*sizeof(char));
    for(i=0; i<len; i++) ret[i]=str[i];
    ret[len]='\0';
    return ret;
}

int AddWindow(PTRWindow* wp)
{
    static int tid=1;
    WindowTable* wt;
    wt=(WindowTable*)malloc(sizeof(WindowTable));
    wt->id=tid;
    wt->wp=wp;
    wt->next=wtable.next;
    wtable.next=wt;
    tid++;
    return wt->id;
}
PTRWindow* IDToWindow(int wid)
{
    WindowTable* wt;
    wt=wtable.next;
    while(wt){
        if(wt->id==wid) return wt->wp;
        wt=wt->next;
    }
    return NULL;
}



int AddData(PTRData* dp)
{
    static int tid=1;
    DataTable* dt;
    if(dp==NULL) return 0;
    dt=(DataTable*)malloc(sizeof(DataTable));
    dt->id=tid;
    dt->dp=dp;
    dt->next=dtable.next;
    dtable.next=dt;
    tid++;
    return dt->id;
}
PTRData* IDToData(int did)
{
    DataTable* dt;
    dt=dtable.next;
    while(dt){
        if(dt->id==did) return dt->dp;
        dt=dt->next;
    }
    return NULL;
}




/* library.c */
void setup_(){
    SetUp(NULL);
}
void pssetup_(){
    PSSetUp();
}

void movewindow_(int* wid, float* x, float* y){
    PTRWindow* wp;
    wp=IDToWindow(*wid);
    MoveWindow(wp, (double)(*x), (double)(*y));
}
void resizewindow_(int* wid, float* width, float* height){
    PTRWindow* wp;
    wp=IDToWindow(*wid);
    ResizeWindow(wp, (double)(*width), (double)(*height));
}
void moveresizewindow_(int* wid, float* x, float* y,
                       float* width, float* height){
    PTRWindow* wp;
    wp=IDToWindow(*wid);
    MoveResizeWindow(wp, (double)(*x), (double)(*y),
                     (double)(*width), (double)(*height));
}
void region_(int* wid, int* rows, int* cols, int* num){
    PTRWindow* wp;
    wp=IDToWindow(*wid);
    WindowRegion(wp, *rows, *cols, *num);
}
void resizeframe_(int* wid, int* wdx, int* wdy){
    ResizeFrame(IDToWindow(*wid),
                (unsigned int)(*wdx), (unsigned int)(*wdy));
}

void createframe_(int* wid, int* gmx, int* gmy,
                  int* wdx, int* wdy, char* title, int len){
    char* tl;
    PTRWindow* wp;
    tl=scopy(title, len);
    wp=CreateFrame(*gmx, *gmy, *wdx, *wdy, tl);
    *wid=AddWindow(wp);
    if(tl) free(tl);
}
void newframe_(int* wid){
    PTRWindow* wp;
    wp=CreateSimpleFrame();
    *wid=AddWindow(wp);
}
void createwindow_(int* pwid, int* wid, float* x, float* y,
                   float* width, float* height){
    PTRWindow* wp;
    wp=CreateWindow(IDToWindow(*pwid), (double)(*x), (double)(*y),
                      (double)(*width), (double)(*height));
    *wid=AddWindow(wp);
}
void newwindow_(int* pwid, int* wid){
    PTRWindow* wp;
    wp=CreateSimpleWindow(IDToWindow(*pwid));
    *wid=AddWindow(wp);
}

void showwindow_(int* wid){
    ShowWindow(IDToWindow(*wid));
}
void showwindows_(int* wid){
    ShowWindows(IDToWindow(*wid));
}

void psland_(char* fname, int len){
    char* fn;
    fn=scopy(fname, len);
    PSLandscape(fname);
    if(fn) free(fn);
}
void psport_(char* fname, int len){
    char* fn;
    fn=scopy(fname, len);
    PSPortrait(fname);
    if(fn) free(fn);
}
void print_(int* wid){
    PSPrint(IDToWindow(*wid));
}
void showpage_(){
    ShowPage();
}
void waitkey_(char* key){
    WaitKeyPress(key, NULL, NULL);
}
void pendingkey_(char* key){
    PendingKeyPress(key, NULL, NULL);
}

/* data.c */
void readgriddata_(int* dim, int* nx1, int* nx2, int* nx3, int* nv,
                   float* x1, float* x2, float* x3, float* v)
{
    int i, j, k;
    double* xx1=NULL;  double* xx2=NULL;
    double* xx3=NULL;  double* vv =NULL;  double** va;
    if(0<(*dim)){
        xx1=(double*)malloc((*nx1)*sizeof(double));
        for(i=0; i<(*nx1); i++) xx1[i]=(double)x1[i];
    }
    if(1<(*dim)){
        xx2=(double*)malloc((*nx2)*sizeof(double));
        for(j=0; j<(*nx2); j++) xx2[j]=(double)x2[j];
    }
    if(2<(*dim)){
        xx3=(double*)malloc((*nx3)*sizeof(double));
        for(k=0; k<(*nx3); k++) xx3[k]=(double)x3[k];
    }
    vv=(double*)malloc((*nx1)*(*nx2)*(*nx3)*(*nv)*sizeof(double));
    for(i=0; i<(*nx1)*(*nx2)*(*nx3)*(*nv); i++) vv[i]=(double)v[i];
    va=(double**)malloc((*nv)*sizeof(double*));
    for(i=0; i<(*nv); i++) va[i]=vv+i*(*nx1)*(*nx2)*(*nx3);
    ReadGridData(*dim, *nx1, *nx2, *nx3, *nv, xx1, xx2, xx3, va);
    if(xx1) free(xx1);  if(xx2) free(xx2);
    if(xx3) free(xx3);  if(vv) free(vv);  if(va) free(va);
}
void readseqdata_(int* dim, int* nx, int* nv,
                  float* x1, float* x2, float* x3, float* v)
{
    int i;
    double* xx1=NULL;  double* xx2=NULL;
    double* xx3=NULL;  double* vv =NULL;  double** va;
    if(0<(*dim)) xx1=(double*)malloc((*nx)*sizeof(double));
    if(1<(*dim)) xx2=(double*)malloc((*nx)*sizeof(double));
    if(2<(*dim)) xx3=(double*)malloc((*nx)*sizeof(double));
    if(0<(*dim)) vv =(double*)malloc((*nx)*(*nv)*sizeof(double));
    for(i=0; i<(*nx); i++){
        if(0<(*dim)) xx1[i]=(double)x1[i];
        if(1<(*dim)) xx2[i]=(double)x2[i];
        if(2<(*dim)) xx3[i]=(double)x3[i];
    }
    for(i=0; i<(*nx)*(*nv); i++) vv[i]=(double)v[i];
    va=(double**)malloc((*nv)*sizeof(double*));
    for(i=0; i<(*nv); i++) va[i]=vv+i*(*nx);
    ReadSeqData(*dim, *nx, *nv, xx1, xx2, xx3, va);
    if(xx1) free(xx1);  if(xx2) free(xx2);
    if(xx3) free(xx3);  if(vv) free(vv);  if(va) free(va);
}
void readdata_(int* nx, float* x1, float* v)
{
    int i;
    double* xx;  double* vv;
    xx=(double*)malloc((*nx)*sizeof(double));
    vv=(double*)malloc((*nx)*sizeof(double));
    for(i=0; i<(*nx); i++){
        xx[i]=(double)x1[i];  vv[i]=(double)v[i];
    }
    ReadData(*nx, xx, vv);
    free(xx);  free(vv);
}
void savedata_(char* fname, int* did, int len){
    SaveData(fname, IDToData(*did));
}
void loaddata_(char* fname, int len){
    LoadData(fname);
}
void tostreamline_(int* fld, int* did, int* l1, int* l2, int* l3,
                   float* x1, float* x2, float* x3, int* num){
    PTRData* dp;
    dp=ToStreamLine(IDToData(*fld), *l1, *l2, *l3, (double)(*x1),
                   (double)(*x2), (double)(*x3), *num);
    *did=AddData(dp);
}

/* graph.c */
/* removegraph(){} */
void clearwindow_(int* wid){
    ClearWindow(IDToWindow(*wid));
}
void clearwindows_(int* wid){
    ClearWindows(IDToWindow(*wid));
}
void drawpoints_(int* wid, int* did, int* l, int* ptype){
    PGP pgp;
    pgp=GetPGP();
    pgp.ptype=*ptype;
    DrawPoints(IDToWindow(*wid), IDToData(*did), *l, pgp);
}
void drawline_(int* wid, int* did, int* l, int* ltype){
    PGP pgp;
    pgp=GetPGP();
    pgp.ltype=*ltype;
    DrawLine(IDToWindow(*wid), IDToData(*did), *l, pgp);
}
void drawmeshlines_(int* wid, int* did, int* l, int* ltype){
    PGP pgp;
    pgp=GetPGP();
    pgp.ltype=*ltype;
    DrawMeshLines(IDToWindow(*wid), IDToData(*did), *l, pgp);
}
void drawstring_(int* wid, float* x1, float* x2,
                 float* x3, float* v,
                 int* flush, int* rot, char* str, int len)
{
    char* s;
    PGP pgp;
    s=scopy(str, len);
    pgp=GetPGP();
    pgp.flush=*flush;  pgp.rot=*rot;
    DrawString(IDToWindow(*wid), (double)(*x1), (double)(*x2),
               (double)(*x3), (double)(*v), s, pgp);
    if(s) free(s);
}
void drawfixstring_(int* wid, float* x, float* y,
                    int* flush, int* rot, char* str, int len)
{
    char* s;
    PGP pgp;
    s=scopy(str, len);
    pgp=GetPGP();
    pgp.flush=*flush;  pgp.rot=*rot;
    DrawFixString(IDToWindow(*wid), (double)(*x), (double)(*y), s, pgp);
    if(s) free(s);
}
void drawcontourline_(int* wid, int* did, int* l, char* dir,
                      int* nn,
                      float* val, int* ltype)
{
    PGP pgp;
    pgp=GetPGP();
    pgp.ltype=*ltype;
    DrawContourLine(IDToWindow(*wid), IDToData(*did), *l,
                    *dir, *nn, (double)(*val), pgp);
}
#ifdef DEBUG
void drawarrows_(int* wid, int* did,
                 int* l1, int* l2, int* l3, float* exp, int* ltype)
{
    PGP pgp;
    pgp=GetPGP();
    pgp.ltype=*ltype;
    DrawArrows(IDToWindow(*wid), IDToData(*did),
               *l1, *l2, *l3, (double)(*exp), pgp);
}
#endif
void removetoplabel_(int* wid){
    RemoveTopLabel(IDToWindow(*wid));
}
void settoplabel_(int* wid, char* str, int len){
    char* s;  s=scopy(str, len);
    SetTopLabel(IDToWindow(*wid), s, GetPGP());
    if(s) free(s);
}
void removex1label_(int* wid){
    RemoveX1Label(IDToWindow(*wid));
}
void setx1label_(int* wid, char* str, int len){
    char* s;  s=scopy(str, len);
    SetX1Label(IDToWindow(*wid), s, GetPGP());
    if(s) free(s);
}
void removex2label_(int* wid){
    RemoveX2Label(IDToWindow(*wid));
}
void setx2label_(int* wid, char* str, int* rot, int len)
{
    char* s;
    PGP pgp;
    s=scopy(str, len);
    pgp=GetPGP();
    pgp.rot=*rot;
    SetX2Label(IDToWindow(*wid), s, pgp);
    if(s) free(s);
}
void removex3plabel_(int* wid){
    RemoveX3Label(IDToWindow(*wid));
}
void setx3label_(int* wid, char* str, int* rot, int len)
{
    char* s;
    PGP pgp;
    s=scopy(str, len);
    pgp=GetPGP();
    pgp.rot=*rot;
    SetX3Label(IDToWindow(*wid), s, pgp);
    if(s) free(s);
}
void removevlabel_(int* wid){
    RemoveVLabel(IDToWindow(*wid));
}
void setvlabel_(int* wid, char* str, int* rot, int len)
{
    char* s;
    PGP pgp;
    s=scopy(str, len);
    pgp=GetPGP();
    pgp.rot=*rot;
    SetVLabel(IDToWindow(*wid), s, pgp);
    if(s) free(s);
}
void removexlabel_(int* wid){
    RemoveXLabel(IDToWindow(*wid));
}
void setxlabel_(int* wid, char* str, int len){
    char* s;  s=scopy(str, len);
    SetXLabel(IDToWindow(*wid), s, GetPGP());
    if(s) free(s);
}
void removeylabel_(int* wid){
    RemoveYLabel(IDToWindow(*wid));
}
void setylabel_(int* wid, char* str, int* rot, int len)
{
    char* s;
    PGP pgp;
    s=scopy(str, len);
    pgp=GetPGP();
    pgp.rot=*rot;
    SetYLabel(IDToWindow(*wid), s, pgp);
    if(s) free(s);
}
void removezlabel_(int* wid){
    RemoveZLabel(IDToWindow(*wid));
}
void setzlabel_(int* wid, char* str, int* rot, int len)
{
    char* s;
    PGP pgp;
    s=scopy(str, len);
    pgp=GetPGP();
    pgp.rot=*rot;
    SetZLabel(IDToWindow(*wid), s, pgp);
    if(s) free(s);
}


/* function.c */
void removescale_(int* wid){
    RemoveScale(IDToWindow(*wid));
}
void setscale_(int* wid){
    SetScale(IDToWindow(*wid), NULL);
}
void exratio_(int* wid, int* val){
    ExactRatio(IDToWindow(*wid), *val);
}
/* setlimit*/
void x1limit_(int* wid, float* x1min, float* x1max){
    SetX1Limit(IDToWindow(*wid),
               (double)(*x1min), (double)(*x1max));
}
void x2limit_(int* wid, float* x2min, float* x2max){
    SetX2Limit(IDToWindow(*wid),
               (double)(*x2min), (double)(*x2max));
}
void x3limit_(int* wid, float* x3min, float* x3max){
    SetX3Limit(IDToWindow(*wid),
               (double)(*x3min), (double)(*x3max));
}
void vlimit_(int* wid, float* vmin, float* vmax){
    SetVLimit(IDToWindow(*wid),
              (double)(*vmin), (double)(*vmax));
}
void xlimit_(int* wid, float* xmin, float* xmax){
    SetXLimit(IDToWindow(*wid),
              (double)(*xmin), (double)(*xmax));
}
void ylimit_(int* wid, float* ymin, float* ymax){
    SetYLimit(IDToWindow(*wid),
              (double)(*ymin), (double)(*ymax));
}
void zlimit_(int* wid, float* zmin, float* zmax){
    SetZLimit(IDToWindow(*wid),
              (double)(*zmin), (double)(*zmax));
}
void setcoord_(int* wid, int* coord){
    SetCoord(IDToWindow(*wid), *coord);
}
void setaxis_(int* wid, int* ax1, int* ax2, int* ax3){
    SetAxis(IDToWindow(*wid), *ax1, *ax2, *ax3);
}


/* util.c */
void raiseframe_(int* wid){
    RaiseFrame(IDToWindow(*wid));
}

/* default.c */
void setcolor_(int* r, int* g, int* b){
    SetColor(*r, *g, *b);
}
void setnamedcolor_(char* name, int len){
    char* nm;  nm=scopy(name, len);
    SetNamedColor(nm);
    if(nm) free(nm);
}
void baselinewidth_(float* width){
    SetBaseLineWidth((double)(*width));
}


/* free.c */


/* error.c */



/* access to struct */

void getnx1_(int* did, int* nx1){
    PTRData* dp;
    if((dp=IDToData(*did))){
        *nx1=dp->nx1;
    }else{
        *nx1=0;
    }
}
void getnx2_(int* did, int* nx2){
    PTRData* dp;
    *nx2=0;
    if((dp=IDToData(*did))){
        if(dp->dim<2) return;
        *nx2=dp->nx2;
    }
}
void getnx3_(int* did, int* nx3){
    PTRData* dp;
    *nx3=0;
    if((dp=IDToData(*did))){
        if(dp->dim<3) return;
        *nx3=dp->nx3;
    }
}
void getx1_(int* did, float* x1, int* num){
    int n=*num;
    PTRData* dp;
    if((dp=IDToData(*did))){
        int i;
        if(x1==NULL) x1=(float*)malloc(*num*sizeof(float));
        if(dp->nx1<*num) n=dp->nx1;
        for(i=0; i<n; i++) x1[i]=(float)(dp->x1[i]);
    }
}
void getx2_(int* did, float* x2, int* num){
    int n=*num;
    PTRData* dp;
    if((dp=IDToData(*did))){
        int j;
        if(dp->dim<2) return;
        if(x2==NULL) x2=(float*)malloc(*num*sizeof(float));
        if(dp->nx2<*num) n=dp->nx2;
        for(j=0; j<n; j++) x2[j]=(float)(dp->x2[j]);
    }
}
void getx3_(int* did, float* x3, int* num){
    int n=*num;
    PTRData* dp;
    if((dp=IDToData(*did))){
        int k;
        if(dp->dim<3) return;
        if(x3==NULL) x3=(float*)malloc(*num*sizeof(float));
        if(dp->nx3<*num) n=dp->nx3;
        for(k=0; k<n; k++) x3[k]=(float)(dp->x3[k]);
    }
}
