#include <stdlib.h>
#include <pgw.h>
#include "ptr.h"



/* refer window properties */

int IsFrameWindow(PTRWindow* wp){
    return IsPGFrame(wp->pgw);
}
PTRWindow* ParentWindow(PTRWindow* wp){
    return wp->parent;
}
PTRWindow* ParentFrame(PTRWindow* wp){
    PTRWindow* parent;
    parent=wp;
    while(!IsFrameWindow(parent))
            parent=ParentWindow(parent);
    return parent;
}

char* AxLabelString(PTRWindow* wp, int ax)
{
    if(wp->label[ax]){
        return wp->label[ax]->str;
    }else{
        return NULL;
    }
}
char* TopLabelString(PTRWindow* wp){
    return AxLabelString(wp, AX_NL);
}
char* X1LabelString(PTRWindow* wp){
    return AxLabelString(wp, AX_X1);
}
char* X2LabelString(PTRWindow* wp){
    return AxLabelString(wp, AX_X2);
}
char* X3LabelString(PTRWindow* wp){
    return AxLabelString(wp, AX_X3);
}
char* VLabelString(PTRWindow* wp){
    return AxLabelString(wp, AX_V);
}
char* XLabelString(PTRWindow* wp){
    return AxLabelString(wp, wp->ax1);
}
char* YLabelString(PTRWindow* wp){
    return AxLabelString(wp, wp->ax2);
}
char* ZLabelString(PTRWindow* wp){
    return AxLabelString(wp, wp->ax3);
}


int HasScale(PTRWindow* wp){
    if(wp->frame){
        return TRUE;
    }else{
        return FALSE;
    }
}

int IsExactRatio(PTRWindow* wp){
    return wp->exact;
}

/* raise */

void RaiseFrame(PTRWindow* wp){
    RaisePGFrame(wp->pgw);
}

/* max min */

int ExtractX1Range(PTRData* data, double* min, double* max)
{
    int i;
    PTRData* dp;
    EnterFunc("ExtractX1Range");
    if(data){
        dp=data;
    }else if(!(dp=CurrentData())){
        SetError(ERROR, E_NoData);
        LeaveFunc();  return FALSE;
    }else if(!(dp->x1)){
        SetError(ERROR, "Data has no x1 data.\n");
        LeaveFunc();  return FALSE;
    }
    *min=dp->x1[0];  *max=dp->x1[0];
    for(i=1; i<dp->nx1; i++){
        if(dp->x1[i]<*min){
            *min=dp->x1[i];
        }else if(*max<dp->x1[i]){
            *max=dp->x1[i];
        }
    }
    LeaveFunc();  return TRUE;
}
int ExtractX2Range(PTRData* data, double* min, double* max)
{
    int i;
    PTRData* dp;
    EnterFunc("ExtractX2Range");
    if(data){
        dp=data;
    }else if(!(dp=CurrentData())){
        SetError(ERROR, E_NoData);
        LeaveFunc();  return FALSE;
    }else if(!(dp->x2)){
        SetError(ERROR, "Data has no x2 data.\n");
        LeaveFunc();  return FALSE;
    }
    *min=dp->x2[0];  *max=dp->x2[0];
    for(i=1; i<dp->nx2; i++){
        if(dp->x2[i]<*min){
            *min=dp->x2[i];
        }else if(*max<dp->x2[i]){
            *max=dp->x2[i];
        }
    }
    LeaveFunc();  return TRUE;
}
int ExtractX3Range(PTRData* data, double* min, double* max)
{
    int i;
    PTRData* dp;
    EnterFunc("ExtractX3Range");
    if(data){
        dp=data;
    }else if(!(dp=CurrentData())){
        SetError(ERROR, E_NoData);
        LeaveFunc();  return FALSE;
    }else if(!(dp->x3)){
        SetError(ERROR, "Data has no x3 data.\n");
        LeaveFunc();  return FALSE;
    }
    *min=dp->x3[0];  *max=dp->x3[0];
    for(i=1; i<dp->nx3; i++){
        if(dp->x3[i]<*min){
            *min=dp->x3[i];
        }else if(*max<dp->x3[i]){
            *max=dp->x3[i];
        }
    }
    LeaveFunc();  return TRUE;
}
int ExtractVRange(PTRData* data, int l, double* min, double* max)
{
    int i, n;
    PTRData* dp;
    EnterFunc("ExtractVRange");
    if(data){
        dp=data;
    }else if(!(dp=CurrentData())){
        SetError(ERROR, E_NoData);
        LeaveFunc();  return FALSE;
    }else if(!(dp->v) || dp->nv<l){
        SetError(ERROR, "Data has no enough v data.\n");
        LeaveFunc();  return FALSE;
    }
    if(dp->type==GRID){
        n=dp->nx1*dp->nx2*dp->nx3;
    }else if(dp->type==SEQ){
        n=dp->nx1;
    }
    *min=dp->v[l-1][0];  *max=dp->v[l-1][0];
    for(i=1; i<n; i++){
        if(dp->v[l-1][i]<*min){
            *min=dp->v[l-1][i];
        }else if(*max<dp->v[l-1][i]){
            *max=dp->v[l-1][i];
        }
    }
    LeaveFunc();  return TRUE;
}
int ExtractAxRange(int ax, PTRData* data, int l,
                   double* min, double* max)
{
    if(ax==AX_X1){
        return ExtractX1Range(data, min, max);
    }else if(ax==AX_X2){
        return ExtractX2Range(data, min, max);
    }else if(ax==AX_X3){
        return ExtractX3Range(data, min, max);
    }else if(ax==AX_V){
        return ExtractVRange(data, l, min, max);
    }else{
        return FALSE;
    }
}
int ExtractXRange(PTRWindow* wp, PTRData* data,
                  int l, double* min, double* max){
    int ret;
    EnterFunc("ExtractXRange");
    ret=ExtractAxRange(wp->ax1, data, l, min, max);
    LeaveFunc();  return ret;
}
int ExtractYRange(PTRWindow* wp, PTRData* data,
                   int l, double* min, double* max){
    int ret;
    EnterFunc("ExtractYRange");
    ret=ExtractAxRange(wp->ax2, data, l, min, max);
    LeaveFunc();  return ret;
}
int ExtractZRange(PTRWindow* wp, PTRData* data,
                   int l, double* min, double* max){
    int ret;
    EnterFunc("ExtractZRange");
    ret=ExtractAxRange(wp->ax3, data, l, min, max);
    LeaveFunc();  return ret;
}
